=== modified file 'drizzled/serialize/replication_event_reader.cc'
--- drizzled/serialize/replication_event_reader.cc	2008-12-06 19:42:46 +0000
+++ drizzled/serialize/replication_event_reader.cc	2008-12-10 02:15:50 +0000
@@ -1,3 +1,6 @@
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <iostream>
 #include <fstream>
 #include <string>
@@ -127,6 +130,7 @@
 int main(int argc, char* argv[])
 {
   GOOGLE_PROTOBUF_VERIFY_VERSION;
+  int file;
 
   if (argc != 2)
   {
@@ -136,17 +140,41 @@
 
   drizzle::EventList list;
 
-  {
-    // Read the existing master.info file
-    fstream input(argv[1], ios::in | ios::binary);
-    if (!list.ParseFromIstream(&input))
-    {
-      cerr << "Failed to parse master.info." << endl;
-      return -1;
-    }
-  }
-
-  printRecord(&list);
+  if ((file= open(argv[1], O_RDONLY)) == -1)
+  {
+    cerr << "Can not open file: " << argv[0] << endl;
+  }
+
+  while (1)
+  {
+    uint64_t length;
+    char *buffer= NULL;
+    char *temp_buffer;
+
+    /* Read the size */
+    if (read(file, &length, sizeof(uint64_t)) != sizeof(uint64_t))
+      break;
+
+    temp_buffer= (char *)realloc(buffer, length);
+    if (temp_buffer == NULL)
+    {
+      cerr << "Memory allocation failure trying to " << length << "."  << endl;
+      exit(1);
+    }
+    buffer= temp_buffer;
+
+    /* Read the record */
+    if (read(file, buffer, length) != length)
+    {
+      cerr << "Could not read entire record." << endl;
+      exit(1);
+    }
+    list.ParseFromArray(buffer, length);
+
+    /* Print the record */
+    printRecord(&list);
+  }
+
 
   return 0;
 }

=== modified file 'drizzled/serialize/replication_event_writer.cc'
--- drizzled/serialize/replication_event_writer.cc	2008-12-06 19:42:46 +0000
+++ drizzled/serialize/replication_event_writer.cc	2008-12-10 02:15:50 +0000
@@ -1,5 +1,9 @@
 #include <iostream>
 #include <fstream>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <string>
 #include <uuid/uuid.h>
 
@@ -109,28 +113,65 @@
   value->add_value("6");
 }
 
+void write_to_disk(int file, drizzle::EventList *list)
+{
+  std::string buffer;
+  uint64_t length;
+  size_t written;
+
+  list->SerializePartialToString(&buffer);
+
+  length= buffer.length();
+
+  cout << "Writing record of " << length << "." << endl;
+
+  if ((written= write(file, &length, sizeof(uint64_t))) != sizeof(uint64_t))
+  {
+    cerr << "Only wrote " << written << " out of " << length << "." << endl;
+    exit(1);
+  }
+
+  if ((written= write(file, buffer.c_str(), length)) != length)
+  {
+    cerr << "Only wrote " << written << " out of " << length << "." << endl;
+    exit(1);
+  }
+}
+
+
 int main(int argc, char* argv[])
 {
   GOOGLE_PROTOBUF_VERIFY_VERSION;
+  int file;
 
-  if (argc != 2) {
+  if (argc != 2) 
+  {
     cerr << "Usage:  " << argv[0] << " REPLICATION_EVENT_LOG " << endl;
     return -1;
   }
 
+  if ((file= open(argv[1], O_APPEND|O_CREAT|O_SYNC|O_WRONLY, S_IRWXU)) == -1)
+  {
+    cerr << "Can not open file: " << argv[0] << endl;
+   exit(0);
+  }
+
   drizzle::EventList list;
 
+  /* Write first set of records */
   write_ddl(list.add_event(), "CREATE TABLE A (a int) ENGINE=innodb");
   write_insert(list.add_event(), transaction_id);
+
+  write_to_disk(file, &list);
+
+  /* Write Second set of records */
+  write_ddl(list.add_event(), "CREATE TABLE A (a int) ENGINE=innodb");
   write_delete(list.add_event(), transaction_id);
   write_update(list.add_event(), transaction_id);
 
-  fstream output(argv[1], ios::out | ios::trunc | ios::binary);
-  if (!list.SerializeToOstream(&output))
-  {
-    cerr << "Failed to write replication event log." << endl;
-    return -1;
-  }
+  write_to_disk(file, &list);
+
+  close(file);
 
   return 0;
 }

=== modified file 'drizzled/serialize/serialize.h'
--- drizzled/serialize/serialize.h	2008-11-09 02:15:03 +0000
+++ drizzled/serialize/serialize.h	2008-12-10 02:15:50 +0000
@@ -8,5 +8,6 @@
 #include <drizzled/serialize/schema.pb.h>
 #include <drizzled/serialize/master_list.pb.h>
 #include <drizzled/serialize/table.pb.h>
+#include <drizzled/serialize/replication_event.pb.h>
 
 #endif /* DRIZZLED_SERIALIZE_SERIALIZE_H */

=== modified file 'plugin/replicator/Makefile.am'
--- plugin/replicator/Makefile.am	2008-12-06 02:02:19 +0000
+++ plugin/replicator/Makefile.am	2008-12-10 02:15:50 +0000
@@ -29,7 +29,6 @@
 
 noinst_LTLIBRARIES=	libreplicator.la
 libreplicator_la_SOURCES=	replicator.cc
-
-EXTRA_LTLIBRARIES=
+libreplicator_la_LDFLAGS = $(top_builddir)/drizzled/serialize/libserialize.la
 
 EXTRA_DIST=

=== modified file 'plugin/replicator/plug.in'
--- plugin/replicator/plug.in	2008-12-08 06:44:37 +0000
+++ plugin/replicator/plug.in	2008-12-10 02:15:50 +0000
@@ -1,4 +1,4 @@
 DRIZZLE_STORAGE_ENGINE(replicator,  replicator, [Replication Engine],
         [Replication Engine for Drizzle])
 DRIZZLE_PLUGIN_STATIC(replicator,   [libreplicator.la])
-#DRIZZLE_PLUGIN_MANDATORY(replicator)  dnl Default
+DRIZZLE_PLUGIN_MANDATORY(replicator)  dnl Default

=== modified file 'plugin/replicator/replicator.cc'
--- plugin/replicator/replicator.cc	2008-12-08 06:44:37 +0000
+++ plugin/replicator/replicator.cc	2008-12-10 02:15:50 +0000
@@ -15,31 +15,111 @@
 
 #define DRIZZLE_SERVER 1 /* for session variable max_allowed_packet */
 #include <drizzled/server_includes.h>
+#include <drizzled/gettext.h>
 #include <drizzled/session.h>
 #include <drizzled/error.h>
 #include <drizzled/item/strfunc.h>
 #include <drizzled/plugin_replicator.h>
-
-static char anchor[100];
-
-static bool statement(Session *, const char *query, size_t query_length)
+#include <drizzled/serialize/serialize.h>
+
+#include <iostream>
+#include <fstream>
+#include <string>
+using namespace std;
+
+static bool isEnabled;
+static char *log_directory= NULL;
+int log_file= -1;
+
+static bool write_to_disk(int file, drizzle::EventList *list)
 {
-  fprintf(stderr, "STATEMENT: %.*s\n", (uint32_t)query_length, query);
+  std::string buffer;
+  uint64_t length;
+  size_t written;
+
+  list->SerializePartialToString(&buffer);
+
+  length= buffer.length();
+
+  cout << "Writing record of " << length << "." << endl;
+
+  if ((written= write(file, &length, sizeof(uint64_t))) != sizeof(uint64_t))
+  {
+    cerr << "Only wrote " << written << " out of " << length << "." << endl;
+    return true;
+  }
+
+  if ((written= write(file, buffer.c_str(), length)) != length)
+  {
+    cerr << "Only wrote " << written << " out of " << length << "." << endl;
+    return true;
+  }
 
   return false;
 }
 
+static bool statement(Session *session, const char *query, size_t)
+{
+  using namespace drizzle;
+
+  drizzle::EventList list;
+
+  if (isEnabled == false)
+    return false;
+  cerr << "Got into statement" <<endl;
+
+  drizzle::Event *record= list.add_event();
+  record->set_type(Event::DDL);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(10);
+  record->set_transaction_id("junk");
+  record->set_schema(session->db);
+  record->set_sql(query);
+
+  return write_to_disk(log_file, &list);
+}
+
 static bool session_init(Session *session)
 {
-  fprintf(stderr, "Starting Session\n");
-  session->setReplicationData(anchor);
+  using namespace drizzle;
+
+  if (isEnabled == false)
+    return false;
+
+  drizzle::EventList *list= new drizzle::EventList;
+  session->setReplicationData(list);
+
+  drizzle::Event *record= list->add_event();
+
+  record->set_type(Event::DDL);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(10);
+  record->set_transaction_id("junk");
+  record->set_schema(session->db);
+  record->set_sql("BEGIN");
 
   return false;
 }
 
 static bool row_insert(Session *session, Table *)
 {
-  fprintf(stderr, "INSERT: %.*s\n", (uint32_t)session->query_length, session->query);
+  using namespace drizzle;
+
+  if (isEnabled == false)
+    return false;
+
+  drizzle::EventList *list= (drizzle::EventList *)session->getReplicationData();
+  drizzle::Event *record= list->add_event();
+
+  record->set_type(Event::INSERT);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(10);
+  record->set_transaction_id("junk");
+  record->set_schema(session->db);
+  record->set_sql(session->query);
 
   return false;
 }
@@ -48,33 +128,82 @@
                           const unsigned char *, 
                           const unsigned char *)
 {
-  fprintf(stderr, "UPDATE: %.*s\n", (uint32_t)session->query_length, session->query);
+  using namespace drizzle;
+
+  if (isEnabled == false)
+    return false;
+
+  drizzle::EventList *list= (drizzle::EventList *)session->getReplicationData();
+  drizzle::Event *record= list->add_event();
+
+  record->set_type(Event::UPDATE);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(10);
+  record->set_transaction_id("junk");
+  record->set_schema(session->db);
+  record->set_sql(session->query);
 
   return false;
 }
 
 static bool row_delete(Session *session, Table *)
 {
-  fprintf(stderr, "DELETE: %.*s\n", (uint32_t)session->query_length, session->query);
+  using namespace drizzle;
+
+  if (isEnabled == false)
+    return false;
+
+  drizzle::EventList *list= (drizzle::EventList *)session->getReplicationData();
+  drizzle::Event *record= list->add_event();
+
+  record->set_type(Event::DELETE);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(10);
+  record->set_transaction_id("junk");
+  record->set_schema(session->db);
+  record->set_sql(session->query);
 
   return false;
 }
 
 static bool end_transaction(Session *session, bool autocommit, bool commit)
 {
+  bool error;
+  using namespace drizzle;
+
+  if (isEnabled == false)
+    return false;
+
+  cerr << "Got into end" <<endl;
+
+  drizzle::EventList *list= (drizzle::EventList *)session->getReplicationData();
+  drizzle::Event *record= list->add_event();
+
+  record->set_type(Event::DELETE);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(10);
+  record->set_transaction_id("junk");
+  record->set_schema(session->db);
+
   if (commit)
   {
     if (autocommit)
-      fprintf(stderr, "COMMIT\n");
+      record->set_sql("COMMIT");
     else
-      fprintf(stderr, "AUTOCOMMIT\n");
+      record->set_sql("AUTOCOMMIT");
   }
   else
-    fprintf(stderr, "ROLLBACK\n");
+    record->set_sql("ROLLBACK");
+
+  error= write_to_disk(log_file, list);
 
   session->setReplicationData(NULL);
+  delete(list);
 
-  return false;
+  return error;
 }
 
 static int init(void *p)
@@ -88,8 +217,56 @@
   repl->row_update= row_update;
   repl->end_transaction= end_transaction;
 
-  return 0;
-}
+
+  if (isEnabled)
+  {
+    using std::string;
+    string logname;
+
+    logname.append(log_directory ? log_directory : "/tmp");
+    logname.append("/replication_log");
+
+    if ((log_file= open(logname.c_str(), O_TRUNC|O_CREAT|O_SYNC|O_WRONLY, S_IRWXU)) == -1)
+    {
+      cerr << "Can not open file: " << logname.c_str() << endl;
+      exit(0);
+    }
+  }
+
+  return 0;
+}
+
+static int deinit(void *)
+{
+  if (log_file != -1)
+    close(log_file);
+
+  return 0;
+}
+
+static DRIZZLE_SYSVAR_BOOL(
+  enabled,
+  isEnabled,
+  PLUGIN_VAR_NOCMDARG,
+  N_("Enable Replicator"),
+  NULL, /* check func */
+  NULL, /* update func */
+  false /* default */);
+
+static DRIZZLE_SYSVAR_STR(
+  directory,
+  log_directory,
+  PLUGIN_VAR_READONLY,
+  N_("Directory to place replication logs."),
+  NULL, /* check func */
+  NULL, /* update func*/
+  NULL /* default */);
+
+static struct st_mysql_sys_var* system_variables[]= {
+  DRIZZLE_SYSVAR(directory),
+  DRIZZLE_SYSVAR(enabled),
+  NULL,
+};
 
 mysql_declare_plugin(replicator)
 {
@@ -100,9 +277,9 @@
   "Basic replication module",
   PLUGIN_LICENSE_GPL,
   init, /* Plugin Init */
-  NULL, /* Plugin Deinit */
+  deinit, /* Plugin Deinit */
   NULL,   /* status variables */
-  NULL,   /* system variables */
+  system_variables,   /* system variables */
   NULL    /* config options */
 }
 mysql_declare_plugin_end;

